#ifndef CGUL_LOG2_H
#define CGUL_LOG2_H

/**
 * log2().  Calculate the floor and ceiling of the base-2 logarithm of
 * <code>unsigned long</code> types.
 *
 * @file
 * @author Paul Serice
 * @brief log2()
 */

#include "cgul_common.h"
#include "cgul_exception.h"

CGUL_BEGIN_C

/**
 * Return the floor of the base-2 logarithm of the number
 * <code>n</code>.  If <code>n</code> is zero, an exception is thrown.
 *
 * @param[in, out] cex c-style exception
 * @param[in] n number
 * @return floor(log2(n))
 */
extern CGUL_EXPORT int
cgul_log2__floor(cgul_exception_t* cex,
                 unsigned long int n);


/**
 * Return the ceiling of the base-2 logarithm of the number
 * <code>n</code>.  If <code>n</code> is zero, an exception is thrown.
 *
 * @param[in, out] cex c-style exception
 * @param[in] n number
 * @return ceil(log2(n))
 */
extern CGUL_EXPORT int
cgul_log2__ceil(cgul_exception_t* cex,
                unsigned long int n);

CGUL_END_C

#endif  /*  CGUL_LOG2_H  */
