#ifndef CGUL_CSV_P_H
#define CGUL_CSV_P_H

#include "cgul_common.h"
#include "cgul_csv.h"
#include "cgul_vector.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_csv</code> class.
 */
struct cgul_csv {

    /**
     * Each record in the csv file is split into fields (or values)
     * and saved in this vector which can be passed back to the user.
     */
    cgul_vector_t fields;

    /**
     * The first record in the csv file often holds the headers that
     * describe each column.  The first time
     * <code>cgul_csv__load_next()</code> is called, it initializes
     * this tree which maps the headers to their positional index.
     * This allows <code>cgul_csv__lookup_by_header()</code> to lookup
     * values using the header instead of the column number.
     */
    cgul_rbtree_t headers;

    /**
     * Interface for reading lines from the CSV file regardless of the
     * EOL convention used in the file or data source. <p>
     */
    cgul_crlf_file_t f;

    /**
     * Whether we own <code>f</code>.
     */
    int we_own_f;

    /**
     * Boolean to keep track of whether the headers have been mapped.
     */
    int are_headers_mapped;

    /**
     * Which line in <code>f</code> are we currently on.
     */
    unsigned long int line_count;

    /**
     * Field separator (comma by default).
     */
    char separator;

};

CGUL_END_C

#endif  /*  CGUL_CSV_P_H  */
