#ifndef CGUL_VECTOR_P_H
#define CGUL_VECTOR_P_H

#include "cgul_common.h"
#include "cgul_vector.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_vector</code> class.
 */
struct cgul_vector {

    /** Dynamically resized array of pointers. */
    void** v;

    /** Count of the number of elements currently held by the vector. */
    size_t size;

    /**
     * Total number of elements the vector can hold before it needs
     * to resize <code>v</code>.
     */
    size_t capacity;

    /**
     * Minimum capacity below which <code>capacity</code> is not
     * allowed to fall.
     */
    size_t minimum_capacity;

};

/**
 * Create a new <code>cgul_vector</code> object having a specific
 * initial <code>capacity</code>.  The caller is responsible for
 * freeing the object by calling <code>cgul_vector__delete()</code>.
 * If memory cannot be allocated, <code>NULL</code> is returned and an
 * exception is thrown.
 *
 * @param[in,out] cex c-style exception
 * @param[in] capacity initial capacity
 * @return new <code>cgul_vector</code> instance
 */
extern CGUL_EXPORT cgul_vector_t
cgul_vector__new_with_capacity(cgul_exception_t* cex,
                               size_t capacity);

CGUL_END_C

#endif  /*  CGUL_VECTOR_P_H  */
