/*
 * This program calculates the greatest common divisor (GCD) of two
 * numbers.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_bcd_t a = NULL;
    cgul_bcd_t b = NULL;
    cgul_bcd_t gcd = NULL;
    cgul_bcd_t lcm = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    a = cgul_bcd__new_from_string(
            &local,
            "87900943853964702237803962033028349457522940351408059603395588921791068274535");
    b = cgul_bcd__new_from_string(
            &local,
            "110394364362159089478540889976879960234485713365560082503118754609776700084535");
    gcd = cgul_bcd__new(&local);
    lcm = cgul_bcd__new(&local);
    if (local) {
        goto out;
    }

    /* Calculate the greatest common divisor (GCD). */
    cgul_bcd__gcd(&local, gcd, a, b);
    if(local) {
        goto out;
    }

    /* Calculate the lowest common multiple (LCM). */
    cgul_bcd__lcm(&local, lcm, a, b);

    /* Print the result. */
    printf("  a = ");
    cgul_bcd__print_as_decimal(&local, a, stdout);
    puts("");

    printf("  b = ");
    cgul_bcd__print_as_decimal(&local, b, stdout);
    puts("");

    printf("gcd = ");
    cgul_bcd__print_as_decimal(&local, gcd, stdout);
    puts("");

    printf("lcm = ");
    cgul_bcd__print_as_decimal(&local, lcm, stdout);
    puts("");

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (lcm) {
        cgul_bcd__delete(lcm);
    }
    if (gcd) {
        cgul_bcd__delete(gcd);
    }
    if (b) {
        cgul_bcd__delete(b);
    }
    if (a) {
        cgul_bcd__delete(a);
    }

    return rv;
}
