#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * Insert the key <code>key</code> into the trie <code>trie</code>.
 * The caller is responsible for calling <code>free()</code> on the
 * key inserted into the trie.  If an error occurs, an exception is
 * thrown.
 *
 * @param[in] trie trie
 * @param[in] key key
 */
static void
main__insert_key(cgul_trie_cxx& trie,
                 const char* key)
{
    char* tmp = NULL;

    try {

        /* Sanity check. */
        if (!key) {
            goto out;
        }
        
        /* Duplicate the key. */
        tmp = cgul_libc_cxx::strdup(key);
        
        /* Insert the key into the trie. */
        if (trie.insert(tmp, NULL)) {
            /* Transfer ownership of the key to the trie. */
            tmp = NULL;
        }

    } catch (...) {
        /* Clean up. */
        if (tmp) {
            free(tmp);
        }
        /* Rethrow. */
        throw;
    }

 out:

    /* Clean up. */
    if (tmp) {
        free(tmp);
    }

}

/**
 * Print the keys in the trie <code>trie</code> in sort order.
 *
 * @param[in] trie trie
 */
static void
main__print_keys(cgul_trie_cxx& trie)
{
    cgul_trie_node_cxx* curr = trie.get_front();
    for ( ; curr ; curr = curr->get_next()) {
        printf("%s\n", curr->get_key());
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_trie_cxx trie;

    try {

        /* Insert keys. */
        main__insert_key(trie, "foo");
        main__insert_key(trie, "bar");
        main__insert_key(trie, "baz");
        
        /* Print the keys in sort order. */
        printf("===== Before call to trie.clear(). \n");
        main__print_keys(trie);
        
        /* Clear the keys. */
        cgul_cache_cxx freer(cgul_cache_cxx::get_freer());
        trie.clear(&freer, NULL);
        
        /* Print the keys in sort order. */
        printf("===== After call to trie.clear(). \n");
        main__print_keys(trie);
        
    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
