/*
 * This program reads all the lines in text file using one call to the
 * following method:
 *
 *     cgul_crlf_file::get_lines()
 *
 * To run this program do the following:
 *
 *     ./main_crlf_file_get_lines_cxx <file>
 */

#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * Print this program's usage message to the file <code>f</code>.  The
 * base name of <code>progname</code> is printed as part of the usage
 * message.
 *
 * @param[in] progname program name
 * @param[in] fout output file
 */
static void
main__print_usage(const char* progname,
                  ostream& ostrm)
{
    char* bname = NULL;

    assert(progname);

    try {

        /* Get the base name for the program. */
        bname = cgul_libc_cxx::basename(progname);

        /* Print the usage. */
        ostrm << endl;
        ostrm << "Usage: " << bname << " <file>" << endl;
        ostrm << endl;

    } catch (...) {
        /* Clean up. */
        if (bname) {
            free(bname);
        }
        throw;
    }

    /* Clean up. */
    if (bname) {
        free(bname);
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    char** lines = NULL;
    unsigned long int line_count = 0;
    cgul_crlf_file_cxx crlf;

    try {

        /* Validate command-line arguments. */
        if (argc != 2) {
            main__print_usage(argv[0], cerr);
            goto out;
        }

        /* Open the file. */
        crlf.open_fname(argv[1], 1 /* is_block_buffered */);

        /* Read all the lines of text from the file with one call. */
        crlf.get_lines(&lines, &line_count);

        /* Print the lines. */
        for (i = 0 ; i < line_count ; ++i) {
            puts(lines[i]);
        }

        /* Clean up. */
        if (lines) {
            for (i = 0 ; i < line_count ; ++i) {
                free(lines[i]);
            }
            free(lines);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

 out:
    return rv;
}
