#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * Compare the integer elements <code>e1</code> and <code>e2</code>
 * returning less than zero, zero, or greater than zero if
 * <code>*e1</code> is less than, equal to, or greater than
 * <code>*e2</code> respectively.
 *
 * @param[in] e1 first element
 * @param[in] e2 second element
 * @return comparison of e1 with e2
 */
int
main__compare_integers(const void* e1,
                       const void* e2)
{
    int rv = 0;
    int int1 = *(int*)e1;
    int int2 = *(int*)e2;

    if (int1 < int2) {
        rv = -1;
    } else if (int1 > int2) {
        rv = 1;
    } else {
        rv = 0;
    }

    return rv;
}

/**
 * Push the integer <code>n</code> onto the heap <code>heap</code>.
 * If an error occurs, an exception is thrown.
 *
 * @param[in] heap heap
 * @param[in] n integer to append
 */
void
main__push_integer(cgul_heap_cxx& heap,
                   int n)
{
    int* buf = NULL;

    try {

        /* Allocate space for a new integer. */
        buf = (int*)cgul_libc_cxx::calloc(1, sizeof(*buf));

        /* Push the integer onto the heap. */
        *buf = n;
        heap.push(buf);

    } catch (...) {

        /* Clean up. */
        if (buf) {
            free(buf);
            buf = NULL;
        }

        /* Rethrow. */
        throw;

    }

    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int* n = 0;
    cgul_heap_cxx heap(main__compare_integers);

    try {

        /* Initialize the heap. */
        for (i = 0 ; i < 10 ; ++i) {
            main__push_integer(heap, i);
        }

        /* Print the top value (without removing it from the heap). */
        n = (int*)heap.top();
        printf("%d\n", *n);
        printf("=====\n");

        /* Print the heap. */
        while (!heap.is_empty()) {
            n = (int*)heap.pop();
            printf("%d\n", *n);
            free(n);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    heap.free_values();

    return rv;
}
