#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    cgul_string_t utf8 = NULL;
    cgul_hstring_t utf16 = NULL;
    cgul_exception_t local = NULL;
    const char* tests[] = {
        "", "f", "   f   ", "foo", "   foo  ", "        ", NULL
    };

    /* Allocate. */
    utf8 = cgul_string__new(&local);
    utf16 = cgul_hstring__new(&local);
    if (local) {
        goto out;
    }

    /* Trim Left: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-16. */
        cgul_hstring__set_value(&local, utf16, CGUL_HCHAR__EMPTY_STRING);
        cgul_unicode__mbstohcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf16);
        /* Trim left. */
        cgul_hstring__trim_left(&local, utf16);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__hcstombs(&local,
                               cgul_hstring__get_value(&local, utf16),
                               cgul_hstring__get_length(&local, utf16),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf(" trim_left(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

    printf("=====\n");

    /* Trim Right: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-16. */
        cgul_hstring__set_value(&local, utf16, CGUL_HCHAR__EMPTY_STRING);
        cgul_unicode__mbstohcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf16);
        /* Trim right. */
        cgul_hstring__trim_right(&local, utf16);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__hcstombs(&local,
                               cgul_hstring__get_value(&local, utf16),
                               cgul_hstring__get_length(&local, utf16),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf("trim_right(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

    printf("=====\n");

    /* Trim: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-16. */
        cgul_hstring__set_value(&local, utf16, CGUL_HCHAR__EMPTY_STRING);
        cgul_unicode__mbstohcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf16);
        /* Trim. */
        cgul_hstring__trim(&local, utf16);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__hcstombs(&local,
                               cgul_hstring__get_value(&local, utf16),
                               cgul_hstring__get_length(&local, utf16),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf("      trim(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (utf16) {
        cgul_hstring__delete(utf16);
    }
    if (utf8) {
        cgul_string__delete(utf8);
    }

    return rv;
}
