/*
 * Test program to compare the performance of incrementing a counter
 * when the counter is an intrinsic type using native addition, an
 * intrinsic type using cgul_defensive__add_size_type(), or a
 * cgul_big_integer.
 *
 * Testing shows that cgul_defensive__add_size_type() is only about
 * twice as slow as native addition, but cgul_big_integer is about 10
 * times slower.
 */

#include "cgul.h"

#define TEST_CGUL_BIG_INT 1
#define TEST_DEFENSIVE_ADD 0
#define TEST_INTRINSIC 0
#define ITERATION_LIMIT (1073741824)

int
main(int argc,
     char* argv[])
{
    int rv = 0;

#if TEST_CGUL_BIG_INT
    cgul_big_integer_t i = NULL;
    cgul_big_integer_t i_max = NULL;
    cgul_big_integer_t one = NULL;
    cgul_exception_t local = NULL;

    puts("\nTesting cgul_big_integer.");

    /* Allocate. */
    i = cgul_big_integer__new(&local);
    one = cgul_big_integer__new_from_long(&local, 1);
    i_max = cgul_big_integer__new_from_long(&local, ITERATION_LIMIT);
    if (local) {
        goto out;
    }

    /* while (i < i_max) */
    while (cgul_big_integer__compare(&local, i , i_max) < 0) {
        cgul_big_integer__add(&local, i, i, one);
        if (local) {
            goto out;
        }
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (i) {
        cgul_big_integer__delete(i);
    }

#elif TEST_DEFENSIVE_ADD

    size_t i = 0;
    size_t i_max = ITERATION_LIMIT;
    cgul_exception_t local = NULL;

    puts("\nTesting cgul_defensive__add_*().");

    for (i = 1 ; i < i_max ; ++i) {
        cgul_defensive__add_size_type(&local, i, 1);
        if (local) {
            goto out;
        }
    }

 out:

#elif TEST_INTRINSIC

    /* Compile with -g to force the compiler to not optimize the loop away. */
    unsigned long int i = 0;
    unsigned long int i_max = ITERATION_LIMIT;

    puts("\nTesting intrisic addition.");

    while (i < i_max) {
        ++i;
    }

#endif

    return rv;
}
