/*
 * This test program loads a file into memory using
 * cgul_libc__load_fname() and immediately writes it back out to disk
 * using cgul_libc__save_fname().  The user specifies both file names
 * on the command line when invoking this program.
 */

#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t block_size = 0;
    char* block = NULL;

    try {

        /* Verify command line. */
        if (argc != 3) {
            fprintf(stderr,
                    "Usage: main_libc_save_file <fname_in> <fname_out>\n");
            goto out;
        }

        /* Load the file into memory. */
        cgul_libc_cxx::load_fname(argv[1], &block, &block_size);

        /* Save the memory to file. */
        cgul_libc_cxx::save_fname(argv[2], block, block_size);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;

    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

 out:

    /* Clean up. */
    if (block) {
        free(block);
    }

    return rv;
}
