#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

static int
print_key(const void* key,
          void* data)
{
    printf("key = %s\n", (char*)key);
    return 1;
}

static int
print_value(void* value,
            void* data)
{
    printf("value = %d\n", *(int*)value);
    return 1;
}

static int
print_pair(const void* key,
           void* value,
           void* data)
{
    printf("key = %s  value = %d\n", (char*)key, *(int*)value);
    return 1;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    char* key = NULL;
    int* value = NULL;
    cgul_multimap_cxx mm((cgul_multimap_cxx::compare_t)strcmp);

    try {

    /* Populate the multimap.  Each key gets two values. */
        for (i = 0 ; i < 10 ; i += 2) {

            /* Allocate a new key and value to add to the tree. */
            key = (char*)calloc(1, sizeof(char) * 2);
            value = (int*)calloc(1, sizeof(*value));
            if (!key || !value) {
                throw runtime_error("out of memory");
            }
            /* Assign values to the newly allocated memory. */
            *key = 'a' + i/2;
            *value = i;
            /* Insert the key/value pair into the tree. */
            mm.insert(key, value, NULL);

            /* Allocate a new value to add to the tree. */
            value = (int*)calloc(1, sizeof(*value));
            if (!value) {
                throw runtime_error("out of memory");
            }
            /* Assign values to the newly allocated memory. */
            *value = i + 1;
            /* Insert the key/value pair into the tree. */
            mm.insert(key, value, NULL);

        }

        /* Print the keys in forward and then reverse order. */
        mm.foldl_keys(print_key, NULL);
        mm.foldr_keys(print_key, NULL);

        puts("=====");

        /* Print the values in forward and then reverse order. */
        mm.foldl_values(0, print_value, NULL);
        mm.foldr_values(0, print_value, NULL);

        puts("=====");

        /* Print the pairs in forward and then reverse order. */
        mm.foldl_pairs(0, print_pair, NULL);
        mm.foldr_pairs(0, print_pair, NULL);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    mm.free_keys();
    mm.free_values();

    return rv;
}
