#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * This is the callback used for
 * <code>cgul_radix_sort_pointers()</code>.  It takes a pointer
 * <code>element</code> to an element of the array of pointers that is
 * being sorted and returns the enumeration associated with that
 * pointer.  In this case, the enumeration is simply the integer to
 * which <code>element</code> points.
 *
 * @param[in] element from the array being sorted
 * @return enumeration for the element
 */
static unsigned long int
main__enumerate(const void* element)
{
    return *(unsigned long int*)element;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    int ascending_order = 1;
    int are_enumerations_signed = 0;
    size_t v_count = 1000000;
    unsigned long int* v = NULL;
    cgul_random_cmwc32_t prng = NULL;
    cgul_exception_t local = NULL;

    try {

        /* Allocate. */
        v = (long unsigned int*)cgul_libc__calloc(&local, v_count, sizeof(*v));
        prng = cgul_random_cmwc32__new(&local);
        if (local) {
            throw cgul_exception_cxx(local);
        }

        /* Initialize. */
        for (i = 0 ; i < v_count ; ++i) {
            //v[i] = v_count / 2 - i - 1;
            v[i] = (unsigned long)cgul_random_cmwc32__next(&local, prng);
        }

        /* Sort the array. */
        cgul_radix_sort_cxx::cgul_radix_sort((void*)v,
                                             v_count,
                                             sizeof(*v) /* element_size */,
                                             ascending_order,
                                             main__enumerate,
                                             are_enumerations_signed);

        /* Verify the list is sorted. */
        for (i = 1 ; i < v_count ; ++i) {
            if (ascending_order) {
                if (v[i - 1] > v[i]) {
                    break;
                }
            } else {
                if (v[i - 1] < v[i]) {
                    break;
                }
            }
        }
    if ((v_count == 0) || (i == v_count)) {
            fprintf(stderr, "Sort verified!\n");
        } else {
            fprintf(stderr, "Sort failed!\n");
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    if (prng) {
        cgul_random_cmwc32__delete(prng);
    }
    if (local) {
        cgul_exception__delete(local);
    }

    return rv;
}
