#include <iomanip>
#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    size_t i_max = 0;
    size_t length = 0;
    cgul_uint32_t hv_rolling = 0;
    cgul_uint32_t hv_substring = 0;
    const char* s = "Hello, Cruel World!  Goodbye, Cruel World!";
    const char* substring = "Goodbye, Cruel World!";

    try {

        /* Determine the length of the substring.  This will be used as
         * the length of the window for the rolling hash. */
        length = strlen(substring);

        /* Directly calculate the hash value for the substring. */
        hv_substring = cgul_rolling_hash_cxx::hash(substring, length);

        /* Print the hash value for the substring. */
        cout << "hv_substring = "
             << setw(10) << hv_substring << endl;

        /* Seed the rolling hash by directly calculating the hash value
         * for the first "length" bytes in s. */
        hv_rolling = cgul_rolling_hash_cxx::hash(s, length);

        /* Print the starting value of the rolling hash. */
        cout << "  hv_rolling = "
             << setw(10) << (unsigned long)hv_rolling << endl;

        /* Calculate the rolling hash until we get to the end of the string. */
        i_max = strlen(s);
        for (i = 0 ; i + length < i_max ; ++i) {
        
            /* Calculate the next hash for the rolling window. */
            hv_rolling = cgul_rolling_hash_cxx::roll(hv_rolling,
                                                     s[i],
                                                     s[i + length],
                                                     length);

            /* Print the current value of the rolling hash. */
            cout << "  hv_rolling = "
                 << setw(10) << (unsigned long)hv_rolling << endl;
        }

        /* Check if the final rolling hash value matches the directly
         * calculated hash value for the substring at the end of the
         * string "s". */
        if (hv_rolling == hv_substring) {
            cout << "Success!" << endl;
        } else {
            cout << "Failure!" << endl;
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;

    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
