#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_wchar_t utf32 = CGUL_WCHAR__NUL;
    char tmp[16] = {0};

    try {

        cgul_block_reader_file_cxx br("tower_of_babel.txt");
        cgul_stream_reader_cxx sr;

        /*
         * NOTE: By default sr decodes UTF-8 with sr.read() returning
         * UTF-32, but sr.accept() encodes the captured text to a
         * UTF-8 string.
         */

        /* Reduce the block size to exercise stitching blocks together. */
        if (!sr.set_block_size(17)) {
            throw cgul_exception_cxx("Unable to change block size.");
        }

        /* Add the block reader. */
        sr.add_block_reader(br);

        for (;;) {

            /* Read the next character from the UTF-8 stream as UTF-32. */
            if (!sr.read(&utf32)) {
                /* EOF */
                break;
            }

            /* Convert the UTF-32 character to a UTF-8 encode sequence. */
            cgul_unicode_cxx::wctomb(utf32, tmp);

            /* Print the UTF-8 encoded sequence. */
            printf("%s", tmp);
            fflush(stdout);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
