#include <iostream>
#include "cgul_cxx.h"

using namespace std;

int main(int argc,
         char* argv[])
{
    int rv = 0;
    const char* line = NULL;
    size_t line_length = 0;

    try {

        /* Create the UTF-8 and UTF-16 string objects. */
        cgul_string_cxx utf8;
        cgul_hstring_cxx utf16;

        /* Open the UTF-8 "tower_of_babel.txt" file. */
        cgul_crlf_file_cxx f("tower_of_babel.txt");

        /* Do NOT strip the byte-order mark (BOM). */
        f.set_strip_utf8_bom(0);

        /* Iterate over each line of f. */
        while ((line = f.get_line()) != NULL) {

            /* Get the number of bytes in "line".  Note that because line
             * is UTF-8 encoded it is not generally the case that the
             * number of characters in "line" will match the number of
             * bytes. */
            line_length = strlen(line);

            /* Convert the line from UTF-8 to UTF-16 and append it to
             * utf16.  Note that the cgul_crlf_file object also has
             * the ability to strip the leading Byte-Order Mark (BOM),
             * but it is disabled above. */
            cgul_unicode_cxx::mbstohcs(line,
                                       line_length,
                                       0, /* skip leading BOM */
                                       utf16);

            /* Add a line break. */
            utf16.append_char((cgul_hchar_t)'\n');
        }

        /* Convert from UTF-16 back to UTF-8.  We can then compare the
         * input and output file of the round-trip conversion. */
        cgul_unicode_cxx::hcstombs(utf16.get_value(),
                                   utf16.get_length(),
                                   0, /* skip_bom */
                                   utf8);

        printf("%s", utf8.get_value());

    } catch (const cgul_exception_cxx e) {
        const char* msg = e.get_message();
        cerr << "*** Error: " << msg << endl;
        rv = 1;
    }

    return rv;
}
